@@ -22,6 +22,15 @@ from openstack import resource
 from openstack import utils
 
 
+# Workaround Python's lack of an undefined sentinel
+# https://python-patterns.guide/python/sentinel-object/
+class Unset:
+    def __bool__(self) -> ty.Literal[False]:
+        return False
+
+
+UNSET: Unset = Unset()
+
 CONSOLE_TYPE_ACTION_MAPPING = {
     'novnc': 'os-getVNCConsole',
     'xvpvnc': 'os-getVNCConsole',
@@ -76,6 +85,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         "vm_state",
         "sort_key",
         "sort_dir",
+        "pinned_availability_zone",
         access_ipv4="access_ip_v4",
         access_ipv6="access_ip_v6",
         has_config_drive="config_drive",
@@ -91,7 +101,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         **tag.TagMixin._tag_query_parameters,
     )
 
-    _max_microversion = '2.91'
+    _max_microversion = '2.96'
 
     #: A list of dictionaries holding links relevant to this server.
     links = resource.Body('links')
@@ -146,6 +156,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: A fault object. Only available when the server status
     #: is ERROR or DELETED and a fault occurred.
     fault = resource.Body('fault')
+    #: The host to boot the server on.
+    host = resource.Body('host')
     #: The host status.
     host_status = resource.Body('host_status')
     #: The hostname set on the instance when it is booted.
@@ -192,6 +204,14 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: networks parameter, the server attaches to the only network
     #: created for the current tenant.
     networks = resource.Body('networks')
+    #: Personality files. This should be a list of dicts with each dict
+    #: containing a file name ('name') and a base64-encoded file contents
+    #: ('contents')
+    personality = resource.Body('personality', type=list)
+    #: The availability zone requested during server creation OR pinned
+    #: availability zone, which is configured using default_schedule_zone
+    #: config option.
+    pinned_availability_zone = resource.Body('pinned_availability_zone')
     #: The power state of this server.
     power_state = resource.Body('OS-EXT-STS:power_state')
     #: While the server is building, this value represents the percentage
@@ -263,7 +283,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         base_path=None,
         **kwargs,
     ):
-        request = super(Server, self)._prepare_request(
+        request = super()._prepare_request(
             requires_id=requires_id,
             prepend_key=prepend_key,
             base_path=base_path,
@@ -318,7 +338,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         exceptions.raise_from_response(response)
         return response
 
-    def change_password(self, session, password):
+    def change_password(self, session, password, *, microversion=None):
         """Change the administrator password to the given password.
 
         :param session: The session to use for making this request.
@@ -326,22 +346,40 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :returns: None
         """
         body = {'changePassword': {'adminPass': password}}
-        self._action(session, body)
+        self._action(session, body, microversion=microversion)
 
-    def get_password(self, session):
+    def get_password(self, session, *, microversion=None):
         """Get the encrypted administrator password.
 
         :param session: The session to use for making this request.
         :returns: The encrypted administrator password.
         """
         url = utils.urljoin(Server.base_path, self.id, 'os-server-password')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='commit')
 
-        response = session.get(url)
+        response = session.get(url, microversion=microversion)
         exceptions.raise_from_response(response)
 
         data = response.json()
         return data.get('password')
 
+    def clear_password(self, session, *, microversion=None):
+        """Clear the administrator password.
+
+        This removes the password from the database. It does not actually
+        change the server password.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
+        url = utils.urljoin(Server.base_path, self.id, 'os-server-password')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='commit')
+
+        response = session.delete(url, microversion=microversion)
+        exceptions.raise_from_response(response)
+
     def reboot(self, session, reboot_type):
         """Reboot server where reboot_type might be 'SOFT' or 'HARD'.
 
@@ -365,14 +403,17 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self,
         session,
         image,
-        name=None,
-        admin_password=None,
-        preserve_ephemeral=None,
-        access_ipv4=None,
-        access_ipv6=None,
-        metadata=None,
-        user_data=None,
-        key_name=None,
+        name=UNSET,
+        admin_password=UNSET,
+        preserve_ephemeral=UNSET,
+        access_ipv4=UNSET,
+        access_ipv6=UNSET,
+        metadata=UNSET,
+        user_data=UNSET,
+        key_name=UNSET,
+        description=UNSET,
+        trusted_image_certificates=UNSET,
+        hostname=UNSET,
     ):
         """Rebuild the server with the given arguments.
 
@@ -391,25 +432,38 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :param metadata: Metadata to set on the updated server. (Optional)
         :param user_data: User data to set on the updated server. (Optional)
         :param key_name: A key name to set on the updated server. (Optional)
+        :param description: The description to set on the updated server.
+            (Optional) (Requires API microversion 2.19)
+        :param trusted_image_certificates: The trusted image certificates to
+            set on the updated server. (Optional) (Requires API microversion
+            2.78)
+        :param hostname: The hostname to set on the updated server. (Optional)
+            (Requires API microversion 2.90)
         :returns: The updated server.
         """
         action = {'imageRef': resource.Resource._get_id(image)}
-        if preserve_ephemeral is not None:
+        if preserve_ephemeral is not UNSET:
             action['preserve_ephemeral'] = preserve_ephemeral
-        if name is not None:
+        if name is not UNSET:
             action['name'] = name
-        if admin_password is not None:
+        if admin_password is not UNSET:
             action['adminPass'] = admin_password
-        if access_ipv4 is not None:
+        if access_ipv4 is not UNSET:
             action['accessIPv4'] = access_ipv4
-        if access_ipv6 is not None:
+        if access_ipv6 is not UNSET:
             action['accessIPv6'] = access_ipv6
-        if metadata is not None:
+        if metadata is not UNSET:
             action['metadata'] = metadata
-        if user_data is not None:
+        if user_data is not UNSET:
             action['user_data'] = user_data
-        if key_name is not None:
+        if key_name is not UNSET:
             action['key_name'] = key_name
+        if description is not UNSET:
+            action['description'] = description
+        if trusted_image_certificates is not UNSET:
+            action['trusted_image_certificates'] = trusted_image_certificates
+        if hostname is not UNSET:
+            action['hostname'] = hostname
 
         body = {'rebuild': action}
         response = self._action(session, body)
@@ -682,7 +736,14 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         body = {"unrescue": None}
         self._action(session, body)
 
-    def evacuate(self, session, host=None, admin_pass=None, force=None):
+    def evacuate(
+        self,
+        session,
+        host=None,
+        admin_pass=None,
+        force=None,
+        on_shared_storage=None,
+    ):
         """Evacuate the server.
 
         :param session: The session to use for making this request.
@@ -690,6 +751,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :param admin_pass: The admin password to set on the evacuated instance.
             (Optional)
         :param force: Whether to force evacuation.
+        :param on_shared_storage: Whether the host is using shared storage.
+            (Optional) (Only supported before microversion 2.14)
         :returns: None
         """
         body: ty.Dict[str, ty.Any] = {"evacuate": {}}
@@ -699,6 +762,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             body["evacuate"]["adminPass"] = admin_pass
         if force is not None:
             body["evacuate"]["force"] = force
+        if on_shared_storage is not None:
+            body["evacuate"]["onSharedStorage"] = on_shared_storage
         self._action(session, body)
 
     def start(self, session):
